import functools
import logging

from typing import Callable

from ppa import ui

from .records import UbuntuServer, DEFAULT_PROMPT


PROMPT_TIMEOUT = 10
CONNECTION_TIMEOUT = 30
DEFAULT_PORT = 22


logger = logging.getLogger(__name__)


def ssh_connection(func: Callable):
    @functools.wraps(func)
    def _ssh_connection(ubuntu_server: UbuntuServer, *args, **kwargs) -> Callable:
        ubuntu_server["base_prompt"] = ubuntu_server.get("base_prompt", DEFAULT_PROMPT)
        ubuntu_server["port"] = ubuntu_server.get("port", DEFAULT_PORT)
        try:
            return func(ubuntu_server, *args, **kwargs)
        except Exception as e:
            if "could not find expected prompt" in str(e).lower():
                logger.error(str(e))
                message = (
                    f"Failed to find SSH prompt matching expected format '{ubuntu_server['base_prompt']}' "
                    f"after {PROMPT_TIMEOUT} seconds. Please view the logs for more information."
                )
                ui.output_error(message)
                raise ValueError(message)
            raise e

    return _ssh_connection
